import bson


class SchedulerBackend:
    def __init__(self):
        """
        Initialize connection to MongoDb backend
        """
        # Check if database and collections exists if not create them
        # list_coll = connection.list_collection_names()
        # for collection in list_coll:
        #     print(collection)

    def create_schedule(self, db_connection, schedule_rec):
        res = db_connection.find_one({"schedule_name": schedule_rec['schedule_name']})
        if res is None:
            res = db_connection.insert_one(schedule_rec)
            return res.inserted_id
        else:
            raise Exception("Schedule Already exists ")

    def disable_schedule(self, db_connection, schedule_id):
        disable_schedule = {"$set": {"schedule_enabled": 0}}
        res = db_connection.update_one(filter={'_id': bson.objectid.ObjectId(schedule_id)},
                                       update=disable_schedule)
        return res.modified_count

    def purge_schedule(self, db_connection, schedule_id):
        res = db_connection.delete_one(filter={'_id': bson.objectid.ObjectId(schedule_id)})
        return res.deleted_count



import datetime
from dateutil import parser
from dateutil.tz import *
from opensearchpy import OpenSearch


class LogHandler:

    def __init__(self, elastic_search_server, elastic_search_port, worker_namespace='test-namespace',
                 index='logstash*'):
        elastic_search_server = elastic_search_server
        elastic_search_port = elastic_search_port
        self.worker_namespace = worker_namespace
        self.search_index = index

        self.client = OpenSearch(
            hosts=[{'host': elastic_search_server, 'port': elastic_search_port}],
            http_compress=True,  # enables gzip compression for request bodies
            use_ssl=False)

    def get_logs(self, schedule_name, output_json=False):
        print("Getting logs ")
        search_query = {"size": 500,
                        "query": {
                            "bool": {
                                "must": [],
                                "filter": [
                                    {
                                        "match_all": {}
                                    },
                                    {
                                        "match_phrase": {
                                            "kubernetes.namespace_name": self.worker_namespace
                                        }
                                    },
                                    {
                                        "match_phrase": {
                                            "kubernetes.pod_name": schedule_name
                                        }
                                    },
                                    # {
                                    #     "range": {
                                    #         "@timestamp": {
                                    #             "gte": "2022-03-13T19:18:22.234Z",
                                    #             "lte": "2022-03-14T10:18:22.234Z",
                                    #             "format": "strict_date_optional_time"
                                    #         }
                                    #     }
                                    # }
                                ],
                                "should": [],
                                "must_not": []
                            }
                        }
                        }

        response = self.client.search(body=search_query, index=self.search_index)
        if output_json:
            json_output = []
            # Returning json output
            for row in response['hits']['hits']:
                json_output.append({"timestamp": parser.parse(row['_source']['@timestamp']).astimezone(tzlocal()),
                                    "pod_name": row['_source']['kubernetes']['pod_name'],
                                    "output": row['_source']['message']
                                    })
            return json_output
        else:
            # Printing output to console
            for row in response['hits']['hits']:
                print("{} - {} - {}".format(parser.parse(row['_source']['@timestamp']).astimezone(tzlocal()),
                                            row['_source']['kubernetes']['pod_name'],
                                            row['_source']['message']))



from pymongo import MongoClient


class ConnectionManager:

    def __init__(self, db_name, **kwargs):
        print("Initiating connection with backend database ")
        mongo_client = MongoClient(**kwargs)
        self.handle = mongo_client[db_name]

    def get_connection(self):
        return self.handle



class ResultBackend:

    def __init__(self):
        pass

    def insert_result_record(self, result_db_collection, rec):
        result_db_collection.insert_one(rec)
        return True

    def get_results(self):
        pass